-- SimpleHUD

-- v1.5 -- added Death Knight power setting (RP)

-- v1.4.2 -- Updated for WoW 3.02 (Lich King, toc 30000)

-- v1.4.1 -- Added: (1) "buff" data type, which is just buff equivalent of "debuff" type.

-- v1.4.0 -- Added: (1) data type "cooldown", which displays cooldown of an actionslot. spell or item must be on an actionslot someplace, not just in a bag or spellbook. this may mean you need to make a macro for it. (2) data type "debuff", which times player-cast debuff on unit. both use new pref field "action".

-- v1.3.1 -- Added (1) if gauge is for energy or combo points, alert color applies ABOVE threshold. this allows alerts to signal sufficient energy or combo for an ability. (2) bugfix for alert threshold activation. wasn't activating at exactly the right point, because of some kind of internal rounding error (floats are NEVER equal!). (3) bugfix for minor issue with druid form color changes. (4) ensured that gauges update immediately after entering combat. (4) bugfix for applying presets to gauges 4 and 5 --- was applying them to gauge 1 instead. (5) streamlined the gauge settings in Waterfall. Now uses a popup to choose which gauge to edit.

-- v1.3.0 25 July 2007 -- added (1) grow to middle option, (2) growth direction option, (3) FOURTH and FIFTH gauges, so can now support two gauges growing to middle in top layer, two to middle in middle layer, one in small layer, (4) 11 segment energy arc, for rogues with Vitality talent

-- v1.2.0 25 July 2007 -- added (1) individual gauge sizes, (2) smart rotation locations

-- v1.10 25 July 2007 -- added (1) unit selection, (2) HUD rotation, (3) xoffset and individual HUD x and y offsets, (4) Waterfall options dialog

-- v1.00 24 July 2007 -- first publish

-- Basic heads-up-display with minimal options, configured how the author likes it.
-- There are FIVE simple segmented arc gauges that display data.
-- Each can display: player/target/pet health, player mana, player energy, player rage, player combo points, or player time in 5 second rule.
-- Each can be hidden and configured separately. Type /shud at prompt for configuration dialog.

-- globals

SimpleHUDversion = "v1.4.2";

prevDataHUD = {-1,-1,-1,-1,-1};   -- holds previous values, so we can check to update only when changed

SimpleHUD_MaxDuration = {0,0,0,0,0}; -- tracks cooldown times

SimpleHUD_HUDFrame = {};
SimpleHUD_HUDTexture = {};

SimpleHUD_OOC = true;

SimpleHUD5secTimer = 0;
SimpleHUDtrackmana = 0;
SimpleHUDalertpulse = 0;
SimpleHUDalertpulseDir = 1;
SimpleHUDalertpulseAmplitude = 1/2; -- seconds/2 in one phase of pulse
SimpleHUDalertFlag = {false,false,false,false,false};

SimpleHUD_editgauge = 1;

-- saved variables, per character
SimpleHUDData = {
    data = {"health","power","combo","health","health"}, -- type of data in each gauge
    unit = {"player","player","player","player","player"}, -- unit to retrieve data from
    action = {"none","none","none","none","none"}, -- ability parameter used by cooldown and debuff
    shown = {true,true,false,false,false}, -- visibility
    shownooc = {true,true,true,true,true}, -- out-of-combat visibility
    color = {{0,1,0},{0,0,1},{1,0,0},{0,1,0},{0,1,0}}, -- gauge color as r,g,b values
    alertcolor = {{1,0,0,0.3},{1,0.5,0,0.3},{1,0,0,0},{1,0,0,0},{1,0,0,0}}, -- alert colors (r,g,b) and data threshold to use
    alertpulse = {true,true,true,true,true},
    opacity = {{1,0.2},{1,0.2},{1,1},{1,1},{1,1}}, -- opacity of HUD from min to max data value
    opacityfull = {0,0,1,1,1}, -- HUD opacity when gauge full
    autocolor = false, -- automatically adjust power colors to power type (for druids)
    yoffset = 0, -- vertical position offset of cluster of HUDs
    xoffset = 0, -- horizontal offset of cluster
    offsets = {{0,0},{0,0},{0,0},{0,0},{0,0}}, -- offset pairs for individual HUDs
    orientation = {0,0,0,0,0}, -- in degrees
    size = {1,2,3,3,3}, -- sizes of gauges: 1=big,3=small
    growclockwise = {true,true,true,true,true}, -- growth direction, clockwise is default
    growmid = {false,false,false,false,false}, -- grow to 50% rather than 100%
    scale = 1, -- overall scaling factor
    shudversion = SimpleHUDversion   -- version mark for pref updating
};

-- Initialize

function SimpleHUD_OnLoad(self)

    -- register events
    
    -- tooltip event catching
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
    
    --- initialization events
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
    self:RegisterEvent("VARIABLES_LOADED");
    
    --- detect combat events
    self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	
	self:RegisterEvent("PLAYER_DEAD");
    
    --- for health data
    self:RegisterEvent("UNIT_HEALTH");
	self:RegisterEvent("UNIT_MAXHEALTH");
	
	--- for power data
	self:RegisterEvent("UNIT_MANA");
	self:RegisterEvent("UNIT_MAXMANA");
	self:RegisterEvent("UNIT_ENERGY");
	self:RegisterEvent("UNIT_RAGE");
	
	--- for combo points
	self:RegisterEvent("UNIT_COMBO_POINTS");
	
	-- for druid shapeshift
	--this:RegisterEvent("PLAYER_AURAS_CHANGED");
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");
	
	-- for target health
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	
	-- for action cooldowns
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	self:RegisterEvent("ACTIONBAR_UPDATE_STATE");
	
	-- for debuff tracking
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_OTHER");
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF");
	self:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF");
	
	-- options init
	SlashCmdList["SHUD"] = SimpleHUD_Options;
	SLASH_SHUD1 = "/shud";

end

-- slash handler
function SimpleHUD_Options(msg)
    
    local param;
    msg,param = strsplit(" ",msg,2);
    
    if msg == "color" then
        -- split param into parts
        local i,r,g,b = strsplit(",",param,4);
        i = tonumber(i);
        r = tonumber(r);
        g = tonumber(g);
        b = tonumber(b);
        SimpleHUDData.color[i] = {r,g,b};
        -- force redraw of HUD
        SimpleHUD_ForceRedraw(i);
    elseif msg == "help" then
        -- print full help
        shprint("SimpleHUD option commands:");
        shprint("  /shud data i,dtype :|r set data type of HUD i to dtype, where dtype may be health, power, combo, or 5secrule");
        shprint("  /shud color i,r,g,b :|r set color of HUD i to r,g,b");
        shprint("  /shud alert i,r,g,b,a :|r set alert color of HUD i to r,g,b, taking effect under proportion a");
        shprint("  /shud offset x,y[,i] :|r set x-offset and y-offset of HUDs to x and y. If HUD index i present, applies only to HUD i.");
        shprint("  /shud show i :|r toggle visibility of HUD i");
        shprint("  /shud showooc i :|r toggle visibility of HUD i while out-of-combat");
        shprint("  /shud opacity i,min,max :|r set opacity of HUD i to range from min to max");
        shprint("  /shud opacityfull i,a :|r set opacity of HUD i at max value to a");
        shprint("  /shud scale s :|r set overall scaling factor to s");
        shprint("  /shud autocolor :|r toggle autocoloring of power (for druids)");
        shprint("  /shud unit i,u :|r set unit of HUD i to u, where u is 'player','target','pet','focus',etc.");
        shprint("  /shud rotate i,a :|r rotate HUD i by a degrees");
        shprint("  /shud preset presetname,i :|r set HUD i to settings in presetname. Type /shud preset (without values) for list of valid presetname values.");
        
    elseif msg == "offset" then
        local x,y,i = strsplit(",",param,3);
        if i == nil then
            SimpleHUDData.yoffset = tonumber(y);
            SimpleHUDData.xoffset = tonumber(x);
        else
            i = tonumber(i);
            SimpleHUDData.offsets[i] = { x , y };
        end
        SimpleHUD_ForceFullRedraw();
    elseif msg == "unit" then
        local i,uid = strsplit(",",param,2);
        i = tonumber(i);
        SimpleHUDData.unit[i] = uid;
        SimpleHUD_ForceRedraw(i);
    elseif msg == "rotate" then
        local i,angle = strsplit(",",param,2);
        i = tonumber(i);
        angle = tonumber(angle);
        SimpleHUDData.orientation[i] = angle;
        SimpleHUD_ForceRedraw(i);
    elseif msg == "show" then
        local i = tonumber(param);
        SimpleHUDData.shown[i] = not SimpleHUDData.shown[i];
        SimpleHUD_ForceRedraw(i);
    elseif msg == "data" then
        local i,dtype = strsplit(",",param,2);
        i = tonumber(i);
        if ( dtype ~= "health" and dtype ~= "power" and dtype ~= "combo" and dtype ~= "5secrule" ) then
            shprint("Invalid data type requested: "..dtype);
            return;
        end
        SimpleHUDData.data[i] = dtype;
        SimpleHUD_ForceRedraw(i);
    elseif msg == "showooc" then
        local i = tonumber(param);
        SimpleHUDData.shownooc[i] = not SimpleHUDData.shownooc[i];
        SimpleHUD_ForceRedraw(i);
    elseif msg == "opacity" then
        local i,mino,maxo = strsplit(",",param,3);
        i = tonumber(i);
        mino = tonumber(mino);
        maxo = tonumber(maxo);
        SimpleHUDData.opacity[i] = { mino , maxo };
        SimpleHUD_ForceRedraw(i);
    elseif msg == "opacityfull" then
        local i,o = strsplit(",",param,2);
        i = tonumber(i);
        o = tonumber(i);
        SimpleHUDData.opacityfull[i] = o;
        SimpleHUD_ForceRedraw(i);
    elseif msg == "scale" then
        SimpleHUDData.scale = tonumber(param);
        SimpleHUD_ForceFullRedraw();
    elseif msg == "autocolor" then
        SimpleHUDData.autocolor = not SimpleHUDData.autocolor;
        local autoout = "OFF";
        if SimpleHUDData.autocolor then autoout = "ON"; end
        shprint("Autocoloring set to "..autoout);
        SimpleHUD_ForceFullRedraw();
    elseif msg == "alert" then
        local i,r,g,b,a = strsplit(",",param,5);
        i = tonumber(i);
        r = tonumber(r);
        g = tonumber(g);
        b = tonumber(b);
        a = tonumber(a);
        SimpleHUDData.alertcolor[i] = { r,g,b,a };
    
    elseif msg == "preset" then
        -- convenient setting presets
        if param == nil then
            -- no parameters, so print list of presets
            shprint("SimpleHUD gauge presets:");
            shprint("  mana,i:|r blue mana gauge, HUD i");
            shprint("  energy,i:|r yellow energy gauge, HUD i");
            shprint("  health,i:|r green health gauge, HUD i");
            shprint("  pethealth,i:|r green pet health gauge, HUD i");
            shprint("  targethealth,i:|r red target health gauge, HUD i");
            shprint("  combo,i:|r red combo point gauge, HUD i");
            shprint("  5secrule,i:|r white 5 second rule gauge, HUD i");
            shprint("SimpleHUD class presets:");
            shprint("  caster:|r sets gauges to health,mana,5secrule");
            shprint("  rogue:|r sets gauges to health,energy,combo");
            shprint("  warrior:|r sets gauges to health,rage,hidden");
            shprint("  druid:|r sets gauges to health,mana,combo, and enables autocoloring for stance changes");
            return;
        end
        local preset,i = strsplit(",",param,2);
        shprint("Loading "..preset.." presets.");
        -- class presets
        if preset == "caster" then
            SimpleHUDData.autocolor = false;
            SimpleHUD_Options("preset health,1");
            SimpleHUD_Options("preset mana,2");
            SimpleHUD_Options("preset 5secrule,3");
        end
        if preset == "rogue" then
            SimpleHUDData.autocolor = false;
            SimpleHUD_Options("preset health,1");
            SimpleHUD_Options("preset energy,2");
            SimpleHUD_Options("preset combo,3");
        end
        if preset == "druid" then
            SimpleHUDData.autocolor = true;
            SimpleHUD_Options("preset health,1");
            SimpleHUD_Options("preset mana,2");
            SimpleHUD_Options("preset combo,3");
        end
        if preset == "warrior" then
            SimpleHUDData.autocolor = false;
            SimpleHUD_Options("preset health,1");
            SimpleHUD_Options("preset rage,2");
            SimpleHUDData.shown[3] = false;
        end
        if preset == "test" then
            SimpleHUDData.autocolor = false;
            SimpleHUD_Options("preset testing,1");
            SimpleHUD_Options("preset testing,2");
            SimpleHUD_Options("preset testing,3");
        end
        i = tonumber(i);
        if preset == "testing" then
            SimpleHUDData.shownooc[i] = true;
            SimpleHUDData.opacity[i] = {0.9,0.9};
            SimpleHUDData.opacityfull[i] = 0.9;
            SimpleHUDData.shown[i] = true; -- make sure visible
        end
        if preset == "mana" then
            SimpleHUDData.data[i] = "power";
            SimpleHUDData.unit[i] = "player";
            SimpleHUDData.shownooc[i] = true;
            SimpleHUDData.color[i] = {0,0,1}; -- blue
            SimpleHUDData.alertcolor[i] = {0.5,0,1,0.4}; -- turn purplish under 40%
            SimpleHUDData.opacity[i] = {1,0.2}; -- fade to transparent as fills
            SimpleHUDData.opacityfull[i] = 0; -- invisible when 100%
            SimpleHUDData.shown[i] = true; -- make sure visible
        end
        if preset == "midclock" then
            SimpleHUDData.shown[i] = true; -- make sure visible
            SimpleHUDData.growmid[i] = true;
            SimpleHUDData.growclockwise[i] = true;
        end
        if preset == "midcounterclock" then
            SimpleHUDData.shown[i] = true; -- make sure visible
            SimpleHUDData.growmid[i] = true;
            SimpleHUDData.growclockwise[i] = false;
        end
        if preset == "fullclock" then
            SimpleHUDData.shown[i] = true; -- make sure visible
            SimpleHUDData.growmid[i] = false;
            SimpleHUDData.growclockwise[i] = true;
        end
        if preset == "fullcounterclock" then
            SimpleHUDData.shown[i] = true; -- make sure visible
            SimpleHUDData.growmid[i] = false;
            SimpleHUDData.growclockwise[i] = fakse;
        end
        if preset == "energy" then
            SimpleHUDData.data[i] = "power";
            SimpleHUDData.unit[i] = "player";
            SimpleHUDData.shownooc[i] = false; -- hide when OOC
            SimpleHUDData.color[i] = {1,1,0}; -- yellow
            SimpleHUDData.alertcolor[i] = {1,1,0,0}; -- disable
            SimpleHUDData.opacity[i] = {0.7,0.7}; -- always bright
            SimpleHUDData.opacityfull[i] = 0.8; -- brightest when full
            SimpleHUDData.shown[i] = true; -- make sure visible
        end
        if preset == "rage" then
            SimpleHUDData.data[i] = "power";
            SimpleHUDData.unit[i] = "player";
            SimpleHUDData.shownooc[i] = true; -- show when OOC
            SimpleHUDData.color[i] = {1,0,0}; -- red
            SimpleHUDData.alertcolor[i] = {1,1,0,0}; -- disable
            SimpleHUDData.opacity[i] = {0.7,0.7}; -- always bright
            SimpleHUDData.opacityfull[i] = 0.7; -- brightest when full
            SimpleHUDData.shown[i] = true; -- make sure visible
        end
        if preset == "health" then
            SimpleHUDData.data[i] = "health";
            SimpleHUDData.unit[i] = "player";
            SimpleHUDData.shownooc[i] = true;
            SimpleHUDData.color[i] = {0,1,0}; -- green
            SimpleHUDData.alertcolor[i] = {1,0,0,0.4}; -- turn red under 40%
            SimpleHUDData.opacity[i] = {1,0.1}; -- fade to transparent as fills
            SimpleHUDData.opacityfull[i] = 0; -- invisible when 100%
            SimpleHUDData.shown[i] = true; -- make sure visible
        end
        if preset == "pethealth" then
            SimpleHUDData.data[i] = "health";
            SimpleHUDData.unit[i] = "pet"; -- unit token for player pet
            SimpleHUDData.shownooc[i] = true;
            SimpleHUDData.color[i] = {0.2,1,0}; -- green, slightly yellow
            SimpleHUDData.alertcolor[i] = {1,0,0,0.4}; -- turn red under 40%
            SimpleHUDData.opacity[i] = {1,0.1}; -- fade to transparent as fills
            SimpleHUDData.opacityfull[i] = 0; -- invisible when 100%
            SimpleHUDData.shown[i] = true; -- make sure visible
        end
        if preset == "targethealth" then
            SimpleHUDData.data[i] = "health";
            SimpleHUDData.unit[i] = "target"; -- unit token for player target
            SimpleHUDData.shownooc[i] = true;
            SimpleHUDData.color[i] = {1,0.2,0}; -- red, slightly orange
            SimpleHUDData.alertcolor[i] = {1,0,0,0.4}; -- turn red under 40%
            SimpleHUDData.opacity[i] = {0.8,0.8}; -- always solid
            SimpleHUDData.opacityfull[i] = 0.8; -- disable
            SimpleHUDData.shown[i] = true; -- make sure visible
        end
        if preset == "5secrule" then
            SimpleHUDData.data[i] = "5secrule";
            SimpleHUDData.unit[i] = "player";
            SimpleHUDData.shownooc[i] = true;
            SimpleHUDData.color[i] = {1,1,1}; -- white
            SimpleHUDData.alertcolor[i] = {1,1,1,0}; -- disable
            SimpleHUDData.opacity[i] = {0.2,0.5}; -- fade as empties
            SimpleHUDData.opacityfull[i] = 0.5; -- same as above
            SimpleHUDData.shown[i] = true; -- make sure visible
        end
        if preset == "combo" then
            SimpleHUDData.data[i] = "combo";
            SimpleHUDData.unit[i] = "player";
            SimpleHUDData.shownooc[i] = true; -- true, but usually zero OOC
            SimpleHUDData.color[i] = {1,0,0}; -- red
            SimpleHUDData.alertcolor[i] = {1,0.5,0,0.99}; -- turn orange under 100%, so orange unless full
            SimpleHUDData.opacity[i] = {0.6,0.6}; -- bright
            SimpleHUDData.opacityfull[i] = 0.8; -- brighter
            SimpleHUDData.shown[i] = true; -- make sure visible
        end
        
        -- after loading presets, refresh frames
        SimpleHUD_ForceFullRedraw();
    
    else
        -- /shud triggers Waterwall
        AceLibrary("Waterfall-1.0"):Open('SimpleHUD');
        
        -- no command caught, so show basic help too
        --shprint("SimpleHUD "..SimpleHUDversion);
        --shprint("  /shud help|r shows full list of commands");
        --shprint("  /shud preset presetname,i|r sets HUD i to settings in presetname.");
        --shprint("  /shud preset|r (without values) for list of valid presetname values.");
    end

end

function SimpleHUD_ForceRedraw(i)
    prevDataHUD[i] = -1;
    SimpleHUD_PrepData(i);
end

function SimpleHUD_ForceFullRedraw()
   SimpleHUD_ForceRedraw(1);
   SimpleHUD_ForceRedraw(2);
   SimpleHUD_ForceRedraw(3);
   SimpleHUD_ForceRedraw(4);
   SimpleHUD_ForceRedraw(5);
end

function shprint(x)
    if x then DEFAULT_CHAT_FRAME:AddMessage("|cffffff00"..x) end
end

-- event handler
function SimpleHUD_OnEvent(self,event,...)

    local arg1,arg2,arg3 = select(3 , ... )

    if event == "PLAYER_ENTERING_WORLD" then

        -- nothing here yet
        
    elseif (event == "UPDATE_MOUSEOVER_UNIT") then
		if (UnitName("mouseover")) then
			--SimpleHUD_UnitFormat("mouseover");
		end
    
    elseif event == "VARIABLES_LOADED" then
        
        -- init Waterfall options
        if AceLibrary:HasInstance("Waterfall-1.0") then
            AceLibrary("Waterfall-1.0"):Register('SimpleHUD',
                'aceOptions', shud_options,
                'title', 'SimpleHUD',
                'treeLevels', 3, 'hideTreeRoot' , false
            );
		end
    
        -- check version of saved variables, in case of previous version
        
        if ( SimpleHUDData.shudversion == nil or SimpleHUDData.shudversion ~= SimpleHUDversion ) then
            if SimpleHUDData.shudversion == nil then
                -- add new fields only for v1.10
                SimpleHUDData.unit = {"player","player","player"};
                SimpleHUDData.xoffset = 0;
                SimpleHUDData.offsets = {{0,0},{0,0},{0,0}};
                SimpleHUDData.orientation = {0,0,0};
                SimpleHUDData.shudversion = "v1.10";
            end
            if SimpleHUDData.shudversion == "v1.10" then
                -- add new fields for v1.2.0
                SimpleHUDData.size = {1,2,3};
                SimpleHUDData.shudversion = "v1.2.0";
            end
            
            if SimpleHUDData.shudversion == "v1.2.0" then
                -- lots changed, so just replace it all
                SimpleHUDData = {
                    data = {"health","power","combo","health","health"},
                    unit = {"player","player","player","player","player"},
                    shown = {true,true,false,false,false},
                    shownooc = {true,true,true,true,true},
                    color = {{0,1,0},{0,0,1},{1,0,0},{0,1,0},{0,1,0}},
                    alertcolor = {{1,0,0,0.3},{1,0.5,0,0.3},{1,0,0,0},{1,0,0,0},{1,0,0,0}},
                    opacity = {{1,0.2},{1,0.2},{1,1},{1,1},{1,1}},
                    opacityfull = {0,0,1,1,1},
                    autocolor = false;
                    yoffset = 0,
                    xoffset = 0,
                    offsets = {{0,0},{0,0},{0,0},{0,0},{0,0}},
                    orientation = {0,0,0,0,0},
                    size = {1,2,3,3,3},
                    growclockwise = {true,true,true,true,true},
                    growmid = {false,false,false,false,false},
                    scale = 1,
                    shudversion = "v1.3.0"
                };
            end
            
            if SimpleHUDData.shudversion == "v1.3.0" then
                -- just update version to 1.3.1
                SimpleHUDData.shudversion = "v1.3.1";
            end
            
            if SimpleHUDData.shudversion == "v1.3.1" then
                -- need to add action field to player prefs
                SimpleHUDData.action = {"none","none","none","none","none"};
                SimpleHUDData.shudversion = "v1.4.0";
            end

            -- should exit with current version stamp
        end
        
        -- initialize
        
        DEFAULT_CHAT_FRAME:AddMessage("|cffB0A0ff<SimpleHUD>|r |cffffff00"..SimpleHUDversion.."|r - ".."/shud for options.");
        
        SimpleHUD_CreateFrames();
        
        SimpleHUDtrackmana = UnitMana("Player");
        
        -- initial display checks
        SimpleHUD_ForceFullRedraw();
    
    elseif ( event == "UNIT_HEALTH" or event == "UNIT_MAXHEALTH"  or event == "PLAYER_TARGET_CHANGED" ) then
        -- check health
        SimpleHUD_PrepScan("health");
        
    elseif ( event == "UNIT_MANA" or event == "UNIT_MAXMANA" or event == "UNIT_ENERGY" or event == "UNIT_RAGE" or event == "PLAYER_TARGET_CHANGED" ) then
    
        -- check if mana dropped
        if SimpleHUDtrackmana > UnitMana("Player") then
            -- start timer
            SimpleHUD5secTimer = 5;
            SimpleHUD_PrepScan("5secrule");
        end
        SimpleHUDtrackmana = UnitMana("Player");
        
        -- check power
        SimpleHUD_PrepScan("power");
        
    elseif event == "UNIT_COMBO_POINTS" then
        -- check combo points
        SimpleHUD_PrepScan("combo");
        
    elseif event == "PLAYER_REGEN_ENABLED" then
        -- out-of-combat flag => on
        SimpleHUD_OOC = true;
    elseif event == "PLAYER_REGEN_DISABLED" then
        -- out-of-combat flag => off
        SimpleHUD_OOC = false;
        SimpleHUD_ForceFullRedraw();
    elseif ( event == "PLAYER_AURAS_CHANGED" or event == "UPDATE_SHAPESHIFT_FORMS" ) then
        -- check for shapeshift and trigger possible autocolor change
        SimpleHUD_ForceFullRedraw();
        SimpleHUD_PrepScan("debuff");
        SimpleHUD_PrepScan("buff");
    elseif ( event == "ACTIONBAR_UPDATE_COOLDOWN" or event == "ACTIONBAR_UPDATE_USABLE" ) then
        -- check for cooldown data
        SimpleHUD_PrepScan("cooldown");
    elseif ( event == "UNIT_SPELLCAST_SUCCEEDED" or event == "CHAT_MSG_SPELL_AURA_GONE_OTHER" or event == "CHAT_MSG_SPELL_AURA_GONE_SELF" or event == "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF" ) then
        -- check if player casting
        if ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then
            if arg1 == "player" then
                SimpleHUD_PrepScan("debuff");
                SimpleHUD_PrepScan("buff");
            end
        else
            SimpleHUD_PrepScan("debuff");
            SimpleHUD_PrepScan("buff");
        end
    end

end

-- check if any gauge needs to prep data type dtype
function SimpleHUD_PrepScan(dtype)

    for i = 1,5 do
        if SimpleHUDData.data[i] == dtype then
            SimpleHUD_PrepData(i);
        end
    end

end

-- make the frames from stored settings
function SimpleHUD_CreateFrames()

    -- make a frame and texture for each gauge
    
    for i = 1,5 do
        -- parent frame
        SimpleHUD_HUDFrame[i] = CreateFrame("Frame","SimpleHUDFrame"..i);
        SimpleHUD_HUDFrame[i]:SetFrameStrata("LOW");
        -- texture
        SimpleHUD_HUDTexture[i] = SimpleHUD_HUDFrame[i]:CreateTexture(nil,"BACKGROUND");
        SimpleHUD_HUDTexture[i]:SetBlendMode("ADD");
        SimpleHUD_HUDTexture[i]:SetAllPoints(SimpleHUD_HUDFrame[i]);
        
        SimpleHUD_HUDFrame[i].baseL = 256;
        SimpleHUD_HUDFrame[i].baseH = 256;
    end

end

-- update frames
-- this does all the drawing
function SimpleHUD_UpdateData(cdata,hudnum)
    
    -- check if we should show this HUD at all
    if not SimpleHUDData.shown[hudnum] then
        SimpleHUD_HUDFrame[hudnum]:Hide();
        return;
    end
    if SimpleHUD_OOC then
        -- out of combat
        if not SimpleHUDData.shownooc[hudnum] then
            SimpleHUD_HUDFrame[hudnum]:Hide();
            return;
        end
    end
    
    -- redraw HUD number hudnum using data cdata
    
    local cstep = 1;
    local cmax = 100;
    local fVitality = false;
    if ( UnitPowerType("player") == 3 and UnitManaMax("player") == 110 ) then
        -- rogue with Vitality, so use 11 segment arc
        fVitality = true;
    end
    
    if ( SimpleHUDData.data[hudnum] ~= "combo" and SimpleHUDData.data[hudnum] ~= "5secrule" ) then
        -- big gauges with 10 steps
        if SimpleHUDData.growmid[hudnum] then
            -- grow to 50%, so only 5 steps
            cstep = floor(cdata/20);
            if cdata <= 5 then cstep = 1; end
            if cdata >= 98 then cstep = 5; end
        else
            -- grow to 100%
            if fVitality then
                -- use 11 segment arc
                cstep = floor(cdata/(100/11));
                if cdata <= 9 then cstep = 1; end
                if cdata >= 98 then cstep = 11; end
            else
                -- use 10 segment arc
                cstep = floor(cdata/10);
                if cdata <= 10 then cstep = 1; end
                if cdata >= 98 then cstep = 10; end
            end
        end
    else
        -- small gauge with 5 steps
        -- cdata should be raw combo points or raw seconds left in 5 sec rule
        -- note that this gauge style ignores growmid option
        cmax = 5;
        if SimpleHUDData.data[hudnum] == "5secrule" then
            -- make it an integer by rounding up 
            cstep = ceil(cdata);
        else
            cstep = cdata;
        end
    end
    
    local theTexture = nil;
    
    if cdata > 0 then
        if cmax == 5 then
            -- use 5-segment arc
            theTexture = "Interface\\Addons\\SimpleHUD\\Textures\\smallArcUpto"..cstep..".tga";
        else
            -- use 10 or 11 segment arc
            if fVitality then
                theTexture = "Interface\\Addons\\SimpleHUD\\Textures\\arc11seg"..cstep..".tga";
            else
                theTexture = "Interface\\Addons\\SimpleHUD\\Textures\\arcUpto"..cstep..".tga";
            end
        end
    end

    -- set texture
    SimpleHUD_HUDTexture[hudnum]:SetTexture(theTexture);
    
    -- ROTATION
    -- 0 degrees : C = 0, F = 0
    -- 90 degrees : 0,1
    -- 270 degrees : 1,0
    -- 180 degrees : 1,1
    
    local rawangle = SimpleHUDData.orientation[hudnum];
    local angle = math.rad( rawangle );
    local A,B,C,D,E,F;
    A = math.cos(angle);
    B = math.sin(angle);
    D = -math.sin(angle);
    E = math.cos(angle);
    C = 0;
    F = 0;
    if rawangle == 270 or rawangle == 180 then C = 1; end
    if rawangle == 90 or rawangle == 180 then F = 1; end
    
	local det = A*E - B*D;
	local ULx, ULy, LLx, LLy, URx, URy, LRx, LRy;
	
	ULx, ULy = ( B*F - C*E ) / det, ( -(A*F) + C*D ) / det;
	LLx, LLy = ( -B + B*F - C*E ) / det, ( A - A*F + C*D ) / det;
	URx, URy = ( E + B*F - C*E ) / det, ( -D - A*F + C*D ) / det;
	LRx, LRy = ( E - B + B*F - C*E ) / det, ( -D + A -(A*F) + C*D ) / det;
	
	if not SimpleHUDData.growclockwise[hudnum] then
	   -- make gauge grow counterclockwise
        if rawangle == 270 then
           -- RIGHT: flip it so it grows upwards
            ULx, LLx, URx, LRx = LLx, ULx, LRx, URx; -- working
        elseif rawangle == 180 then
            -- BOTTOM: flip it so it grows rightwards
            ULx, LLx, URx, LRx = URx, LRx, ULx, LLx; -- working
        elseif rawangle == 90 then
            -- LEFT: flip it so it grows downwards
            ULx, LLx, URx, LRx = LLx, ULx, LRx, URx; -- working
        elseif rawangle == 0 then
            -- TOP: flip it so it grows leftwards
            ULx, LLx, URx, LRx = URx, LRx, ULx, LLx; -- working
        end
    end
        
	SimpleHUD_HUDTexture[hudnum]:SetTexCoord(ULx, ULy, LLx, LLy, URx, URy, LRx, LRy);
	
    -- compute opacity, interpolated
    local maxO = SimpleHUDData.opacity[hudnum][2]; -- opacity when cdata at max
    local minO = SimpleHUDData.opacity[hudnum][1]; -- opacity when cdata at min
    local fO = cdata/cmax;
    local theO = (1-fO)*minO + fO*maxO;
    if fO == 1 then
        theO = SimpleHUDData.opacityfull[hudnum];
    end
    
    -- alert?
    SimpleHUDalertFlag[hudnum] = false;
    if ( SimpleHUDData.data[hudnum] == "power" and UnitPowerType("player") == 3 ) or ( SimpleHUDData.data[hudnum] == "combo" ) then
        -- energy or combo points, so alert applies ABOVE threshold
        if floor(fO*100) >= floor(SimpleHUDData.alertcolor[hudnum][4]*100) then
            SimpleHUDalertFlag[hudnum] = true; end
    else
        -- alert applies BELOW threshold, in all other cases
        if floor(fO*100) <= floor(SimpleHUDData.alertcolor[hudnum][4]*100) then
            SimpleHUDalertFlag[hudnum] = true; end
    end
    
    -- set color
    local r = SimpleHUDData.color[hudnum][1];
    local g = SimpleHUDData.color[hudnum][2];
    local b = SimpleHUDData.color[hudnum][3];
    if SimpleHUDalertFlag[hudnum] then
        -- use alert color and opacity pulse
        r = SimpleHUDData.alertcolor[hudnum][1];
        g = SimpleHUDData.alertcolor[hudnum][2];
        b = SimpleHUDData.alertcolor[hudnum][3];
        SimpleHUD_HUDFrame[hudnum]:SetAlpha( theO );
        --local pulseO = (SimpleHUDalertpulse + SimpleHUDalertpulseAmplitude)/(2 * SimpleHUDalertpulseAmplitude)/1.25 + 0.2; -- from 0.2 to 1
        --SimpleHUD_HUDFrame[hudnum]:SetAlpha( pulseO );
    else
        -- regular gradient opacity
        SimpleHUD_HUDFrame[hudnum]:SetAlpha( theO );
    end
    if ( SimpleHUDData.autocolor and SimpleHUDData.data[hudnum] == "power" and SimpleHUDData.unit[hudnum] == "player" ) then
        -- adjust color for power type, because druids can switch types
        local pt = UnitPowerType("player");
        if pt == 0 then
            -- mana
            r,g,b = 0,0,1;
        elseif pt == 1 then
            -- rage
            r,g,b = 1,0,0;
        elseif pt == 3 then
            -- energy
            r,g,b = 1,1,0;
        end
    end
    
    -- new death knight health coloring by presence
    if ( SimpleHUDData.autocolor and SimpleHUDData.data[hudnum] == "health" and SimpleHUDData.unit[hudnum] == "player" ) then
        -- adjust color for presence
        local _, _, p1, _ = GetShapeshiftFormInfo(1)
        local _, _, p2, _ = GetShapeshiftFormInfo(2)
        local _, _, p3, _ = GetShapeshiftFormInfo(3)
        if p1 == 1 then
            -- blood
            r,g,b = 1,0,0;
        elseif p2 == 1 then
            -- frost
            r,g,b = 0.8,0.8,1;
        else
            -- unholy
            r,g,b = 0,1,0;
        end
    end
    
    SimpleHUD_HUDTexture[hudnum]:SetVertexColor( r , g , b );
    
    -- positioning and scaling
    --- adjust for both size of gauge and rotation
    
    local HUDypos = 200;
    local HUDxpos = 0;
    local HUDscale = 1;
    
    -- default top position, arc crests on top
    if SimpleHUDData.orientation[hudnum] == 0 then
        -- default size == 1
        HUDypos = 200;
        HUDxpos = 0;
        HUDscale = 1;
        if SimpleHUDData.size[hudnum] == 2 then
            -- middle gauge
            HUDypos = 200 - (200 - 169)*SimpleHUDData.scale;
            HUDscale = 0.76;
        elseif SimpleHUDData.size[hudnum] == 3 then
            -- small gauge
            HUDypos = 200 - (200 - 145)*SimpleHUDData.scale;
            HUDxpos = 1 * SimpleHUDData.scale;
            HUDscale = 0.57;
        end
    end -- angle 0
    
    -- arc crests on LEFT, so place on left
    if SimpleHUDData.orientation[hudnum] == 90 then
        -- default size == 1
        HUDypos = 0;
        HUDxpos = -200;
        HUDscale = 1;
        if SimpleHUDData.size[hudnum] == 2 then
            -- middle gauge
            HUDypos = 0;
            HUDxpos = -200 - (-200 + 169)*SimpleHUDData.scale;
            HUDscale = 0.76;
        elseif SimpleHUDData.size[hudnum] == 3 then
            -- small gauge
            HUDypos = 0 * SimpleHUDData.scale;
            HUDxpos = -200 - (-200 + 145)*SimpleHUDData.scale;
            HUDscale = 0.57;
        end
    end -- angle 90
    
    -- arc crests on RIGHT, so place on right
    if SimpleHUDData.orientation[hudnum] == 270 then
        -- default size == 1
        HUDypos = 0;
        HUDxpos = 200;
        HUDscale = 1;
        if SimpleHUDData.size[hudnum] == 2 then
            -- middle gauge
            HUDypos = 0;
            HUDxpos = 200 - (200 - 169)*SimpleHUDData.scale;
            HUDscale = 0.76;
        elseif SimpleHUDData.size[hudnum] == 3 then
            -- small gauge
            HUDypos = 0 * SimpleHUDData.scale;
            HUDxpos = 200 - (200 - 145)*SimpleHUDData.scale;
            HUDscale = 0.57;
        end
    end -- angle 270
    
    -- arc crests on BOTTOM, so place on bottom
    if SimpleHUDData.orientation[hudnum] == 180 then
        -- default size == 1
        HUDypos = -200;
        HUDxpos = 0;
        HUDscale = 1;
        if SimpleHUDData.size[hudnum] == 2 then
            -- middle gauge
            HUDxpos = 0;
            HUDypos = -200 - (-200 + 169)*SimpleHUDData.scale;
            HUDscale = 0.76;
        elseif SimpleHUDData.size[hudnum] == 3 then
            -- small gauge
            HUDxpos = 0 * SimpleHUDData.scale;
            HUDypos = -200 - (-200 + 145)*SimpleHUDData.scale;
            HUDscale = 0.57;
        end
    end -- angle 180
    
    -- add global and individual offsets
    local thisxoffset = SimpleHUDData.xoffset + SimpleHUDData.offsets[hudnum][1];
    local thisyoffset = SimpleHUDData.yoffset + SimpleHUDData.offsets[hudnum][2];
    
    -- finally position
    SimpleHUD_HUDFrame[hudnum]:SetPoint("Center", HUDxpos + thisxoffset , HUDypos + thisyoffset );
	SimpleHUD_HUDFrame[hudnum]:SetWidth(SimpleHUD_HUDFrame[hudnum].baseL * HUDscale * SimpleHUDData.scale );
	SimpleHUD_HUDFrame[hudnum]:SetHeight(SimpleHUD_HUDFrame[hudnum].baseH * HUDscale * SimpleHUDData.scale );
	
	SimpleHUD_HUDFrame[hudnum]:Show();
    
end

-- compute data
function SimpleHUD_PrepData(hudnum)
    
    local cdata = 0;
    local theunit = SimpleHUDData.unit[hudnum];
    
    if SimpleHUDData.data[hudnum] == "health" then
        cdata = 100 * UnitHealth(theunit)/UnitHealthMax(theunit);
    elseif SimpleHUDData.data[hudnum] == "power" then
        cdata = 100 * UnitMana(theunit)/UnitManaMax(theunit);
        --lmprint(cdata)
    elseif SimpleHUDData.data[hudnum] == "combo" then
        cdata = GetComboPoints("player","target");
    elseif SimpleHUDData.data[hudnum] == "5secrule" then
        cdata = SimpleHUD5secTimer;
    elseif SimpleHUDData.data[hudnum] == "cooldown" then
        cdata = 100 * SimpleHUD_GetActionCooldown(SimpleHUDData.action[hudnum],hudnum);
    elseif SimpleHUDData.data[hudnum] == "debuff" then
        cdata = 100 * SimpleHUD_GetDebuffExpire(SimpleHUDData.action[hudnum],hudnum);
    elseif SimpleHUDData.data[hudnum] == "buff" then
        cdata = 100 * SimpleHUD_GetBuffExpire(SimpleHUDData.action[hudnum],hudnum);
    end
    
    if prevDataHUD[hudnum] ~= cdata then
        -- new value, so redraw
        prevDataHUD[hudnum] = cdata;
        SimpleHUD_UpdateData(cdata,hudnum);
    end
    
end

function SimpleHUD_GetDebuffExpire(adebuff, hudnum)
    -- scan debuffs on unit[hudnum] and return time remaining on one that matches adebuff
    local lExpire = 0;
    local lSlot = 0;
    for lSlot = 1, 40 do
        local name, rank, texture, count, debuffType, duration, timeLeft  =  UnitDebuff(SimpleHUDData.unit[hudnum], lSlot);
        if name then
            if name == adebuff then
                lExpire = timeLeft;
                if not lExpire then
                    -- expired
                    SimpleHUD_MaxDuration[hudnum] = .1;
                    return 0;
                end
                if lExpire > SimpleHUD_MaxDuration[hudnum] then
                    SimpleHUD_MaxDuration[hudnum] = lExpire;
                end
                return lExpire/SimpleHUD_MaxDuration[hudnum]; -- return time left in seconds divided by max duration
            end
        end
    end
    SimpleHUD_MaxDuration[hudnum] = .1; -- reset so we don't get in trouble if action in this gauge changes
    return 0; -- didn't find it
end

function SimpleHUD_GetBuffExpire(adebuff, hudnum)
    -- scan Buffs on unit[hudnum] and return time remaining on one that matches adebuff
    local lExpire = 0;
    local lSlot = 0;
    for lSlot = 1, 40 do
        local name, rank, texture, count, duration, timeLeft  =  UnitBuff(SimpleHUDData.unit[hudnum], lSlot);
        if name then
            if name == adebuff then
                lExpire = timeLeft;
                if not lExpire then
                    -- expired
                    SimpleHUD_MaxDuration[hudnum] = .1;
                    return 0;
                end
                if lExpire > SimpleHUD_MaxDuration[hudnum] then
                    SimpleHUD_MaxDuration[hudnum] = lExpire;
                end
                return lExpire/SimpleHUD_MaxDuration[hudnum]; -- return time left in seconds divided by max duration
            end
        end
    end
    SimpleHUD_MaxDuration[hudnum] = .1; -- reset so we don't get in trouble if action in this gauge changes
    return 0; -- didn't find it
end

function SimpleHUD_GetActionCooldown(aname, hudnum)
    -- scan all action slots and return cooldown on action with name == aname
    local lActionSlot = 0;
    local foundSlot = nil;
	for lActionSlot = 1, 120 do
		local lActionText = GetActionText(lActionSlot);
		if lActionText then
            if lActionText == aname then
                foundSlot = lActionSlot;
                break
            end
		end
	end
	
	if foundSlot == nil then return 0 end -- didn't find it, so give zero data
	
	local start, duration, enable = GetActionCooldown(foundSlot);
    if ( start == 0 ) then
        -- not active, so return ready state (zero)
        SimpleHUD_MaxDuration[hudnum] = 0; -- reset so we don't get in trouble if action in this gauge changes
        return 0;
    else
        -- cooldown is underway, so return state
        if duration > SimpleHUD_MaxDuration[hudnum] then
            SimpleHUD_MaxDuration[hudnum] = duration;
        end
        local remaining = SimpleHUD_ComputeCooldownRemaining(start,duration);
        if remaining == 0 then SimpleHUD_MaxDuration[hudnum] = 0; end
        return remaining/SimpleHUD_MaxDuration[hudnum];
    end
	
end

function SimpleHUD_ComputeCooldownRemaining(start,duration)
    return duration - ( GetTime() - start);
end

function SimpleHUD_OnUpdate(self,elapsed)

    if not elapsed then elapsed = 0 end
    
    -- handle tracking 5-sec rule
    
    if SimpleHUD5secTimer > 0 then
        --shprint(SimpleHUD5secTimer);
        SimpleHUD5secTimer = SimpleHUD5secTimer - elapsed;
        if SimpleHUD5secTimer < 0 then
            SimpleHUD5secTimer = 0;
        end
        SimpleHUD_PrepScan("5secrule");
    end
    
    -- handle cooldown and debuff tracking
    for i = 1,5 do
        if SimpleHUD_MaxDuration[i] > 0 then
            -- cooldown or debuff underway, so update
            SimpleHUD_PrepData(i);
        end
    end
    
    SimpleHUD_PrepScan("power")
    
    for hudnum = 1,5 do
        if not InCombatLockdown() then
            -- out of combat
            if not SimpleHUDData.shownooc[hudnum] then
                SimpleHUD_HUDFrame[hudnum]:Hide()
            end
        end
    end -- hudnum
    
    -- handle alert pulse
    --[[
    if SimpleHUDalertpulseDir > 0 then
        --up
        SimpleHUDalertpulse = SimpleHUDalertpulse + elapsed;
    else
        --down
        SimpleHUDalertpulse = SimpleHUDalertpulse - elapsed;
    end
    if abs(SimpleHUDalertpulse) > SimpleHUDalertpulseAmplitude then
        -- change direction
        SimpleHUDalertpulse = SimpleHUDalertpulseAmplitude * SimpleHUDalertpulseDir;
        SimpleHUDalertpulseDir = -SimpleHUDalertpulseDir;
    end
    
    -- redraw pulsing gauges
    for i = 1,5 do
        if SimpleHUDalertFlag[i] then
            SimpleHUD_ForceRedraw(i);
        end
    end
    ]]--
    
end

-- toy tooltip flavor text
function SimpleHUD_UnitFormat(unit)

	local name = UnitName(unit);
	
	if (name == "Dagma" and UnitClass(unit)=="Priest") then
        GameTooltip:AddLine("\"Beards over Babies\"");
    elseif (name == "Lightpockets" and UnitClass(unit)=="Rogue") then
        GameTooltip:AddLine("\"D.I.C.K.\"");
    elseif (name == "Zeven" and UnitClass(unit)=="Warrior") then
        GameTooltip:AddLine("\"Benevolent Dictator\"");
    elseif (name == "Sirenna" and UnitClass(unit)=="Warlock") then
        GameTooltip:AddLine("\"The power behind the throne\"");
    elseif (name == "Raysa" and UnitClass(unit)=="Druid") then
        GameTooltip:AddLine("\"The power behind the throne\"");
    elseif (name == "Nizoral" and UnitClass(unit)=="Warlock") then
        GameTooltip:AddLine("\"Kills the fungus that causes evil.\"");
    elseif (name == "Ninali" and UnitClass(unit)=="Priest") then
        GameTooltip:AddLine("\"/faly\"");
    elseif (name == "Jesserie" and UnitClass(unit)=="Warrior") then
        GameTooltip:AddLine("\"la la\"");
    elseif (name == "Ardilla" and UnitClass(unit)=="Priest") then
        GameTooltip:AddLine("\"Sciurus umbra\"");
	end
	GameTooltip:Show();
end

-- define options for Waterfall

gaugepresetlist = { ["health"] = "Health", ["mana"] = "Mana", ["combo"] = "Combo points", ["energy"] = "Energy", ["5secrule"] = "Five second rule", ["targethealth"] = "Target's health", ["pethealth"] = "Pet's health", ["fullclock"] = "Full clockwise", ["fullcounterclock"] = "Full counterclockwise", ["midclock"] = "Half clockwise", ["midcounterclock"] = "Half counterclockwise" };

gaugedatatypes = { ["health"] = "Health", ["power"] = "Power", ["combo"] = "Combo points", ["5secrule"] = "Five second rule timer" , ["cooldown"] = "Action cooldown" , ["debuff"] = "Debuff expire" , ["buff"] = "Buff expire" };

shud_options = {
    type = 'group',
    name = 'SimpleHUD',
    desc = 'Top level options',
    args = {
        globals = {
            type = 'group',
            name = 'Global settings',
            desc = 'Settings that affect all gauges',
            args = {
                offsetxG = {
                    type = 'range',
                    name = 'Horizontal position',
                    desc = "X-axis offset for the HUD",
                    get = function() return SimpleHUDData.xoffset end,
                    set = function(v) SimpleHUDData.xoffset = v; SimpleHUD_ForceFullRedraw(); end,
                    min = -400,
                    max = 400,
                    step = 1,
                    isPercent = false,
                    order = 98
                },
                offsetyG = {
                    type = 'range',
                    name = 'Vertical position',
                    desc = "Y-axis offset for the HUD",
                    get = function() return SimpleHUDData.yoffset end,
                    set = function(v) SimpleHUDData.yoffset = v; SimpleHUD_ForceFullRedraw(); end,
                    min = -400,
                    max = 400,
                    step = 1,
                    isPercent = false,
                    order = 99
                },
                overallscale = {
                    type = 'range',
                    name = 'HUD scale',
                    desc = "Scale of all gauges",
                    get = function() return SimpleHUDData.scale end,
                    set = function(v) SimpleHUDData.scale = v; SimpleHUD_ForceFullRedraw(); end,
                    min = 0.2,
                    max = 2,
                    step = 0.1,
                    isPercent = false,
                    order = 100
                },
                showHUD1 = {
                    type = 'toggle',
                    name = 'Autocolor power gauges',
                    desc = "Automatically adjust color of power gauges for druid forms",
                    get = function() return SimpleHUDData.autocolor end,
                    set = function(v) SimpleHUDData.autocolor = v; SimpleHUD_ForceFullRedraw(); end,
                    map = { [false] = "Off", [true] = "On" },
                    order = 101
                },
            } -- end global args
        } -- end globals
        ,
        HUD1 = {
            type = 'group',
            name = 'Gauge settings',
            desc = 'Settings for each individual gauge',
            args = {
                gaugenum = {
                    type = 'text',
                    name = 'Gauge number to edit',
                    desc = "Select which gauge to customize",
                    get = function() return tostring(SimpleHUD_editgauge) end,
                    set = function(v) SimpleHUD_editgauge = tonumber(v); AceLibrary("Waterfall-1.0"):Refresh('SimpleHUD'); end,
                    validate = { ["1"] = "Gauge 1", ["2"] = "Gauge 2", ["3"] = "Gauge 3", ["4"] = "Gauge 4", ["5"] = "Gauge 5" },
                    order = 80
                },
                showHUD1 = {
                    type = 'toggle',
                    name = 'Show gauge',
                    desc = "Toggle visibility of gauge",
                    get = function() return SimpleHUDData.shown[SimpleHUD_editgauge] end,
                    set = function(v) SimpleHUDData.shown[SimpleHUD_editgauge] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    map = { [false] = "Hidden", [true] = "Shown" },
                    order = 88
                },
                showHUD1ooc = {
                    type = 'toggle',
                    name = 'Show gauge out-of-combat',
                    desc = "Toggle out-of-combat visibility of gauge",
                    get = function() return SimpleHUDData.shownooc[SimpleHUD_editgauge] end,
                    set = function(v) SimpleHUDData.shownooc[SimpleHUD_editgauge] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    map = { [false] = "Hidden", [true] = "Shown" },
                    order = 89
                },
                presets1 = {
                    type = 'text',
                    name = 'Load gauge presets',
                    desc = "A set of presets to apply to the gauge",
                    get = function() return "choose..." end,
                    set = function(v) SimpleHUD_Options("preset "..v..","..SimpleHUD_editgauge); SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    validate = gaugepresetlist,
                    order = 90
                },
                data1 = {
                    type = 'text',
                    name = 'Data type',
                    desc = "Type of data to display",
                    get = function() return SimpleHUDData.data[SimpleHUD_editgauge] end,
                    set = function(v) SimpleHUDData.data[SimpleHUD_editgauge] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    validate = gaugedatatypes,
                    order = 91
                },
                unit1 = {
                    type = 'text',
                    name = 'Data source',
                    desc = "Unit to fetch data from",
                    get = function() return SimpleHUDData.unit[SimpleHUD_editgauge] end,
                    set = function(v) SimpleHUDData.unit[SimpleHUD_editgauge] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    validate = { ["player"] = "Player (self)", ["target"] = "Your target", ["focus"] = "Your focus", ["pet"] = "Your pet" },
                    order = 92
                },
                action1 = {
                    type = 'text',
                    name = 'Action',
                    desc = "Cooldown or buff/debuff to track",
                    get = function() return SimpleHUDData.action[SimpleHUD_editgauge] end,
                    set = function(v) SimpleHUDData.action[SimpleHUD_editgauge] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    order = 92
                },
                angle1 = {
                    type = 'text',
                    name = 'Orientation',
                    desc = "Rotation of the gauge",
                    get = function() return tostring(SimpleHUDData.orientation[SimpleHUD_editgauge]) end,
                    set = function(v) SimpleHUDData.orientation[SimpleHUD_editgauge] = tonumber(v); SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    validate = { ["0"] = "Top (default)", ["90"] = "Left", ["270"] = "Right", ["180"] = "Bottom" },
                    order = 120
                },
                color1 = {
                    type = 'color',
                    name = 'Gauge color',
                    desc = "Gauge color",
                    get = function() return SimpleHUDData.color[SimpleHUD_editgauge][1], SimpleHUDData.color[SimpleHUD_editgauge][2], SimpleHUDData.color[SimpleHUD_editgauge][3] end,
                    set = function(r,g,b) SimpleHUDData.color[SimpleHUD_editgauge] = {r,g,b}; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    order = 93
                },
                alertcolor1 = {
                    type = 'color',
                    name = 'Alert color',
                    desc = "Gauge color under/above alert threshold.",
                    get = function() return SimpleHUDData.alertcolor[SimpleHUD_editgauge][1], SimpleHUDData.alertcolor[SimpleHUD_editgauge][2], SimpleHUDData.alertcolor[SimpleHUD_editgauge][3] end,
                    set = function(r,g,b) SimpleHUDData.alertcolor[SimpleHUD_editgauge] = {r,g,b,SimpleHUDData.alertcolor[SimpleHUD_editgauge][4]}; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    order = 94
                },
                alertvalue1 = {
                    type = 'range',
                    name = 'Alert threshold',
                    desc = "Percent at which to use alert color. Energy and combo point gauges enter alert above this amount. All other gauges enter alert below it.",
                    get = function() return SimpleHUDData.alertcolor[SimpleHUD_editgauge][4] end,
                    set = function(v) SimpleHUDData.alertcolor[SimpleHUD_editgauge][4] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    min = 0,
                    max = 1,
                    step = 0.1,
                    isPercent = true,
                    order = 95
                },
                opacitymin1 = {
                    type = 'range',
                    name = 'Opacity gradient minimum',
                    desc = "Opacity when gauge at zero",
                    get = function() return SimpleHUDData.opacity[SimpleHUD_editgauge][1] end,
                    set = function(v) SimpleHUDData.opacity[SimpleHUD_editgauge][1] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    min = 0,
                    max = 1,
                    step = 0.1,
                    isPercent = true,
                    order = 97
                },
                opacitymax1 = {
                    type = 'range',
                    name = 'Opacity gradient maximum',
                    desc = "Opacity when gauge is full",
                    get = function() return SimpleHUDData.opacity[SimpleHUD_editgauge][2] end,
                    set = function(v) SimpleHUDData.opacity[SimpleHUD_editgauge][2] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    min = 0,
                    max = 1,
                    step = 0.1,
                    isPercent = true,
                    order = 98
                },
                opacityfull1 = {
                    type = 'range',
                    name = 'Opacity at 100%',
                    desc = "Opacity when gauge is full. This overrides the gradient.",
                    get = function() return SimpleHUDData.opacityfull[SimpleHUD_editgauge] end,
                    set = function(v) SimpleHUDData.opacityfull[SimpleHUD_editgauge] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    min = 0,
                    max = 1,
                    step = 0.1,
                    isPercent = true,
                    order = 96
                },
                offsetx1 = {
                    type = 'range',
                    name = 'Horizontal position',
                    desc = "X-axis offset for this gauge",
                    get = function() return SimpleHUDData.offsets[SimpleHUD_editgauge][1] end,
                    set = function(v) SimpleHUDData.offsets[SimpleHUD_editgauge][1] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    min = -400,
                    max = 400,
                    step = 1,
                    isPercent = false,
                    order = 99
                },
                offsety1 = {
                    type = 'range',
                    name = 'Vertical position',
                    desc = "Y-axis offset for this gauge",
                    get = function() return SimpleHUDData.offsets[SimpleHUD_editgauge][2] end,
                    set = function(v) SimpleHUDData.offsets[SimpleHUD_editgauge][2] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    min = -400,
                    max = 400,
                    step = 1,
                    isPercent = false,
                    order = 99
                },
                gaugesize = {
                    type = 'range',
                    name = 'Size',
                    desc = "Relative position and scale of gauge in HUD",
                    get = function() return SimpleHUDData.size[SimpleHUD_editgauge] end,
                    set = function(v) SimpleHUDData.size[SimpleHUD_editgauge] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    min = 1,
                    max = 3,
                    step = 1,
                    isPercent = false,
                    order = 121
                },
                growmid1 = {
                    type = 'toggle',
                    name = 'Grow to middle',
                    desc = "Toggle gauge growing to 50% location instead of 100%",
                    get = function() return SimpleHUDData.growmid[SimpleHUD_editgauge] end,
                    set = function(v) SimpleHUDData.growmid[SimpleHUD_editgauge] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    map = { [false] = "Grow normal", [true] = "Grow to middle" },
                    order = 122
                },
                growclockwise1 = {
                    type = 'toggle',
                    name = 'Grow clockwise',
                    desc = "Toggle gauge growing clockwise/counterclockwise",
                    get = function() return SimpleHUDData.growclockwise[SimpleHUD_editgauge] end,
                    set = function(v) SimpleHUDData.growclockwise[SimpleHUD_editgauge] = v; SimpleHUD_ForceRedraw(SimpleHUD_editgauge); end,
                    map = { [false] = "Counterclockwise", [true] = "Clockwise" },
                    order = 123
                }
            } -- end HUD1 args
        } -- end HUD1
        
    } -- end root args
};